;Flying wiggler
;by smkdan

;USES EXTRA BIT.
;If extra bit is set, wiggler will be angered if Mario gets within a distance of it specified by Extra prop 2

;chases Mario when his head is stomped.  Will chase Mario like Phanto until he is either lost or his chasing time expires.

;Extra Property Byte 1: Time before wiggler gives up chasing Mario.
;Extra Property Byte 2: Distance to anger wiggler if proximity mode is on.  See Extra bit.

;Tilemaps for SP4

DIR = $03
PROPRAM = $04
PROPINDEX = $05
XDISPINDEX = $06
PRIINDEX = $07
FRAMEINDEX = $08
TEMP = $09

;This specifies how long to wait before beginning the chase.  Only 00-7F are valid.
STALL_TIME = $20

SINE	dcb $00,$F4,$EA,$E3,$E0,$E3,$EA,$F4 
	dcb $00,$0C,$16,$1D,$20,$1D,$16,$0C

INCDEC	dcb $01,$FF

;CHASE VALUES
;Byte 4 should be the inverse of byte 1 (1)
;Byte 2 should be the inverse of byte 3 (2)

;Bytes 4 and 1 are used when wiggler is chasing mario in the same direction he is travelling. (LESS)
;Bytes 2 and 3 are used when wiggler is chasing Mario but travelling in a different direction. (MORE)
;The net effect is that wiggler will have his speed biased (MORE) when he needs to turn around, otherwise he is very easy to lose.

ADDSUB	dcb $01,$FD,$03,$FF

EORTBL	dcb $00,$FF


;=====

dcb "INIT"
	JSR SUB_HORZ_POS
	TYA
	STA $157C,x		;make it face mario initially
	STZ $1570,x		;reset frame counter TEMP: decision bits for GFX

	LDA #STALL_TIME
	STA $C2,x		;reset counter that's decremented when it's angry
	RTL

dcb "MAIN"
	PHB
	PHK
	PLB
	JSR Run
	PLB
	RTL	;also nothing


Return_l
	RTS

Return_I
	JSL $01802A		;update speed
	RTS
	
Run
	JSR SUB_OFF_SCREEN_HB
	JSR GET_DRAW_INFO
	JSR GFX

	LDA $14C8,x		;02 when dieing?
	CMP #$08            	 
	BNE Return_l           
	LDA $9D			;locked sprites?
	BNE Return_l
	LDA $15D0,x		;being eaten by yoshi?
	BNE Return_l
	
	JSL $01A7DC		;mario interact
	BCC Check_Proximity	;If not interacting, see if proximity interaction is activated
        JSR SUB_VERT_POS 
        LDA $0E
        CMP #$F0                ;larger value means mario can jump into the side of its head
        BPL Hurt_Mario
        LDA $7D			;don't interact on rising speed
        BMI No_Interaction

;Mario bounce. Make wiggler angry

	LDA $1490		;star status, check first
	BNE Star_Kill

	LDA $151C,x		;decision bit in GFX routine to make his GFX angry
	ORA #$01
	STA $151C,x

	JSL $01AB99             ;contact graphic
        LDA #$13                ;sound effect
        STA $1DF9

	LDA #$A8
	STA $7D			;new mario Yspd
	BRA No_Interaction

Check_Proximity
	LDA $7FAB10,x
	AND #$04		;extra bit set?
	BEQ No_Interaction	;skip to no interaction if clear

	JSR PROXIMITY		;proximity routine, Z = 0 means its in range
	BEQ Out_of_range

	LDA $151C,x		;decision bit in GFX routine to make his GFX angry
	ORA #$01
	STA $151C,x
	
Out_of_range
	BRA No_Interaction	;NOTHING FOR NOW

Hurt_Mario
	LDA $1490		;star status checked first
	BNE Star_Kill

	JSL $00F5B7		;hurt
	BRA No_Interaction

Star_Kill
	JSR HAS_STAR		;star kill routine that's been borrowed
	JMP Skip2		;just update speed

No_Interaction
	LDA $151C,x		;if bit 0 clear, it's not angry
	AND #$01
	BNE Angry
	JMP SineW 
Angry
	LDA $C2,x		;behaviour bit
	AND #$80		;stall decision bit, don't chase yet if not set
	BEQ Dont_Chase_Yet

	LDA $C2,x		;else it's set to chase...
	AND #$7F		;clip decision bit set for chasing
	BEQ Stop_Chasing	;if zero, stop chasing.  Just go to smoke routine
	
;Chase code

	JSR SUB_HORZ_POS
	TYA			;direction 00/01 into A
	BIT $1528,x		;test high bit of stored speed
	BPL Going_Right		;high bit set? / going right?
	ORA #$02		;set bit 1 optionally

Going_Right
	TAY			;A to serve as index
	LDA ADDSUB,y		;load appropriate additive
	CLC
	ADC $1528,x
	STA $1528,x
	STA $B6,x		;new X spd

	JSR SUB_VERT_POS
	TYA
	BIT $1602,x
	BPL Going_Up
	ORA #$02

Going_Up
	TAY
	LDA ADDSUB,y
	CLC
	ADC $1602,x
	STA $1602,x
	STA $AA,x		;new Y spd
	
	LDA $14			;frame counter...
	AND #$0F
	BNE Smoke		;only decrement counter every 16th frame
	
	DEC $C2,x		;decrement new decision byte

	BRA Smoke

Dont_Chase_Yet
	STZ $AA,x		;freeze, no X or Y spd
	STZ $B6,x

	DEC $C2,x		;decrement chase counter...
	BNE Smoke		;if not yet zero, move along

	LDA $7FAB28,x		;Extra Prop 1
	LSR A			;drop low bit
	ORA #$80		;new decision bit (CHASE) plus extra property byte
	STA $C2,x
	BRA Smoke		;skip to Smoke routine

Stop_Chasing
	LDA #$00
	LDY $1528,x		;don't modify the stored values so it can just go offscreen
	BMI SCGoing_Left	;going right?
	LDA #$01		;set bit 1
SCGoing_Left
	TAY
	LDA $1528,x
	CLC
	ADC INCDEC,y		;whatever will make it lose X speed

	STA $1528,x		;storage
	STA $B6,x		;new Xspd

	LDA $1602,x
	CMP #$8F
	BEQ Dont_IncY
	DEC $1602,x		;make it fly upwards
Dont_Incy
	STA $AA,x		;new Yspd

	BRA Skip2		;not changing it's direction	
		
Smoke	
	LDA $14			;show smoke every 16 frames
	AND #$0F
	BNE No_Smoke

	JSR SUB_SMOKE

No_Smoke		
	JSR SUB_HORZ_POS
	TYA
	STA $157C,x		;ONLY CHANGE DIRECTION UNTIL CHANGE BEGINS

	BRA Skip2

SineW
	JSR Sine_Motion		;do the wave

Skip2
	JSL $01802A		;update speed

Return
	RTS	

;====

Sine_Motion
	LDA #$08		;fixed speed for normal travel
	LDY $157C,x		;load direction...
	EOR EORTBL,y		;invert accordingly
	STA $B6,x		;Xspd

	LDA $14			;frame
	LSR A
	LSR A			;every 8th frame, change
	LSR A
	AND #$0F		;only 16 entries
	TAY
	LDA SINE,y		;grab entry
	STA $AA,x		;new Yspd

	RTS
;GFX
;===

;=====

TILEMAP	dcb $E6
	dcb $E0,$C6,$E0
	dcb $CC

	dcb $E6
	dcb $E2,$C6,$E2
	dcb $CC

	dcb $E6
	dcb $E4,$C6,$E4
	dcb $CC

	dcb $E6
	dcb $E2,$C6,$E2
	dcb $CC

;...
	dcb $E0,$C8,$E0
	dcb $CC

	dcb $E2,$C8,$E2
	dcb $CC

	dcb $E4,$C8,$E4
	dcb $CC

	dcb $E2,$C8,$E2
	dcb $CC

XDISP	dcb $08
	dcb $FA,$00,$0E
	dcb $08

	dcb $07
	dcb $FA,$00,$0E
	dcb $08

	dcb $06
	dcb $FA,$00,$0E
	dcb $08

	dcb $07
	dcb $FA,$00,$0E
	dcb $08

;...

	dcb $FA,$00,$0E
	dcb $08

	dcb $FA,$00,$0E
	dcb $08

	dcb $FA,$00,$0E
	dcb $08

	dcb $FA,$00,$0E
	dcb $08



YDISP	dcb $F8
	dcb $FE,$FF,$FD
	dcb $09

	dcb $F7
	dcb $FE,$00,$FD
	dcb $09

	dcb $F7
	dcb $FE,$01,$FD
	dcb $09

	dcb $F7
	dcb $FE,$00,$FD
	dcb $09

;..
	dcb $FE,$FF,$FD
	dcb $09

	dcb $FE,$00,$FD
	dcb $09

	dcb $FE,$FF,$FD
	dcb $09

	dcb $FE,$00,$FD
	dcb $09

PROP	dcb $00
	dcb $40,$00,$00
	dcb $00

;...

	dcb $40,$00,$00
	dcb $00


PRI	dcb $04
	dcb $03,$00,$02	;priority list: actually an OAM index 
	dcb $01

;...
	dcb $03,$00,$02
	dcb $01

PALSWAP	dcb $18,$14

EORF	dcb $40,$00

;IF HIGH BIT OF $C2,X IS SET, AND THE LOW 7 BITS ARE 0, FLASH PALETTE!

GFX
	LDA $15F6,x	;load properties
	STA PROPRAM	;into TEMP

	LDA $157C,x
	STA DIR		;convenience for the direction

	LDA $14
	AND #$03	;every 4th frame
	BNE Skip
	LDA $9D		;due to odd way it animates, skip animating if sprites are locked
	BNE Skip
	LDA $1570,x	;grab frame no.
	INC A
	STA $1570,x
	CMP #$04	;4 = overflow
	BNE Skip
	STZ $1570,x
	
Skip
	LDA $151C,x	;angry wiggler?
	AND #$01
	BEQ Not_Angry
	LDA $1570,x	;frame no...
	ASL A
	ASL A		;x4, only 4 bytes per entry for this one
	CLC
	ADC #$14	;add 20, skip past happy frames
	STA FRAMEINDEX

	LDA #$05	;property index and priority index past 5 non angry bytes
	STA PROPINDEX
	STA PRIINDEX

	LDA $C2,x	;chaser status
	CMP #$80	;if bit7 is set and bits 0-6 (counter) are spent
	BNE Not_Flashing;solid red palette if not flashing

;!!
	LDA $14		;frame counter...
	LSR A		;every X frame, switch palette
	LSR A
	AND #$01	;just bit0
	PHX		;preserve...
	TAX		;into X
	LDA PROPRAM
	AND #$01	;only page bit
	ORA PALSWAP,x	;OR with one of two palettes, yellow and red
	STA PROPRAM	;new propram
	PLX		;restore
	BRA Skip1	;skip past non angry section
	
;!!
	
Not_Flashing
	LDA PROPRAM
	AND #$01	;only leave the page bit
	ORA #$18	;0C << 1
	STA PROPRAM	;new properties
	
	BRA Skip1	;skip past the not angry setup

Not_Angry	
	LDA $1570,x
	ASL A		;x2
	ASL A		;x4
	CLC
	ADC $1570,x	;x5
	STA FRAMEINDEX	;5 bytes per entry
	STZ PROPINDEX
	STZ PRIINDEX

Skip1
	STY TEMP	;store OAM index in TEMP...
	LDX #$00	;reset loop index

OAM_Loop
	TXA
	CLC
	ADC PRIINDEX	;load priortiy index
	PHX
	TAX
	LDA PRI,x	
	ASL A		;x4, 4 bytes per sprite
	ASL A
	CLC
	ADC TEMP	;add original OAM index
	TAY		;into Y, new OAM index
	PLX

	TXA
	CLC
	ADC FRAMEINDEX
	PHX
	TAX
	LDA DIR
	BNE XLeft

	LDA $00
	SEC
	SBC XDISP,x
	STA $0300,y
	PLX
	BRA XNext
XLeft
	LDA $00
	CLC
	ADC XDISP,x
	STA $0300,y
	PLX

XNext
	TXA	
	CLC
	ADC FRAMEINDEX
	PHX
	TAX
	LDA $01
	CLC
	ADC YDISP,x
	STA $0301,y
	PLX

	TXA
	CLC
	ADC FRAMEINDEX
	PHX
	TAX
	LDA TILEMAP,x
	STA $0302,y
	PLX

	TXA
	CLC
	ADC PROPINDEX
	PHX
	TAX
	LDA PROPRAM	;sprite properties
	ORA PROP,x	;flip/mirror bits
	ORA $64		;level info
	PHX	
	LDX DIR		;load direction
	EOR EORF,x	;invert mirror as needed
	PLX
	STA $0303,y
	PLX

	INX
	PHX
	LDX $15E9
	LDA $151C,x
	AND #$01
	BEQ Not_Angry1
	PLX
	CPX #$04	;only four tiles for angry one
	BNE OAM_Loop
	
	LDX $15E9	;reload sprite index

	LDY #$02
	LDA #$03	;4 tiles
	JSL $01B7B3

	RTS		;return

Not_Angry1
	PLX		;preserve previously pushed loop index
	CPX #$05	;5 tiles
	BEQ Loop_End
	JMP OAM_Loop

Loop_End
	LDX $15E9	;restore sprite index
	
	LDY #$02
	LDA #$04	;5 tiles
	JSL $01B7B3

	RTS

;=====
;a routine to calculate whether or not Mario is within range of wiggler.  Based off fang.asm
;Z = 0 when in range, 1 when out of range

EORTBLI	dcb $FF,$00
		
PROXIMITY
	LDA $14E0,x		;sprite x high
	XBA
	LDA $E4,x		;sprite x low
	REP #$20		;16bitA
	SEC
	SBC $94			;sub mario x
	SEP #$20		;8bitA
	PHA			;preserve for routine jump
	JSR SUB_HORZ_POS	;horizontal distance
	PLA			;restore
	EOR EORTBLI,y		;invert if needed
	CMP $7FAB34,x		;range define by Extra Prop 2*
	BCS PRange_Out		;return not within range
	LDA #$01		;Z = 0
	RTS

PRange_Out
	LDA #$00		;Z = 1
	RTS
	
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; a smoke routine I mangled for angry wiggler's head
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

SUB_SMOKE           LDY #$03                ; \ find a free slot to display effect
FINDFREE1           LDA $17C0,y             ;  |
                    BEQ FOUNDONE1            ;  |
                    DEY                     ;  |
                    BPL FINDFREE1            ;  |
                    RTS                     ; / return if no slots open

FOUNDONE1           LDA #$01                ; \ set effect graphic to smoke graphic
                    STA $17C0,y             ; /
                    LDA $D8,x               ; \ smoke y position = generator y position
                    CLC
                    ADC #$F4
                    STA $17C4,y             ; /
                    LDA #$0E                ; \ set time to show smoke
                    STA $17CC,y             ; /
                    LDA $E4,x               ; \ load generator x position and store it for later
		    CLC
		    ADC #$F0
                    STA $17C8,y             ; /
                    LDX $15E9 

                    LDY #$03                ; \ find a free slot to display effect
FINDFREE2           LDA $17C0,y             ;  |
                    BEQ FOUNDONE2            ;  |
                    DEY                     ;  |
                    BPL FINDFREE2            ;  |
                    RTS                     ; / return if no slots open

FOUNDONE2           LDA #$01                ; \ set effect graphic to smoke graphic
                    STA $17C0,y             ; /
                    LDA $D8,x               ; \ smoke y position = generator y position
                    CLC
                    ADC #$F4
                    STA $17C4,y             ; /
                    LDA #$0E                ; \ set time to show smoke
                    STA $17CC,y             ; /
                    LDA $E4,x               ; \ load generator x position and store it for later
		    CLC
		    ADC #$0C
                    STA $17C8,y             ; /
                    LDX $15E9
             
                    RTS


;=================
;BORROWED ROUTINES
;=================

STAR_SOUNDS         dcb $00,$13,$14,$15,$16,$17,$18,$19

KILLED_X_SPEED      dcb $F0,$10

HAS_STAR            LDA #$02                ; \ status = 2 (being killed by star)
                    STA $14C8,x             ; /
                    LDA #$D0                ; \ set y speed
                    STA $AA,x               ; /
                    JSR SUB_HORZ_POS        ; get new direction
                    LDA KILLED_X_SPEED,y    ; \ set x speed based on direction
                    STA $B6,x               ; /
                    INC $18D2               ; increment number consecutive enemies killed
                    LDA $18D2               ; \
                    CMP #$08                ;  | if consecutive enemies stomped >= 8, reset to 8
                    BCC NO_RESET2           ;  |
                    LDA #$08                ;  |
                    STA $18D2               ; /   
NO_RESET2           JSL $02ACE5             ; give mario points
                    LDY $18D2               ; \ 
                    CPY #$08                ;  | if consecutive enemies stomped < 8 ...
                    BCS NO_SOUND2           ;  |
                    LDA STAR_SOUNDS,y       ;  |    ... play sound effect
                    STA $1DF9               ; /
NO_SOUND2           RTS                     ; final return

;GET_DRAW_INFO
;==============

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; $B760 - graphics routine helper - shared
; sets off screen flags and sets index to OAM
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

SPR_T1              dcb $0C,$1C
SPR_T2              dcb $01,$02

GET_DRAW_INFO       STZ $186C,x             ; reset sprite offscreen flag, vertical
                    STZ $15A0,x             ; reset sprite offscreen flag, horizontal
                    LDA $E4,x               ; \
                    CMP $1A                 ;  | set horizontal offscreen if necessary
                    LDA $14E0,x             ;  |
                    SBC $1B                 ;  |
                    BEQ ON_SCREEN_X         ;  |
                    INC $15A0,x             ; /

ON_SCREEN_X         LDA $14E0,x             ; \
                    XBA                     ;  |
                    LDA $E4,x               ;  |
                    REP #$20                ;  |
                    SEC                     ;  |
                    SBC $1A                 ;  | mark sprite invalid if far enough off screen
                    CLC                     ;  |
                    ADC.W #$0040            ;  |
                    CMP.W #$0180            ;  |
                    SEP #$20                ;  |
                    ROL A                   ;  |
                    AND #$01                ;  |
                    STA $15C4,x             ; / 
                    BNE INVALID             ; 
                    
                    LDY #$00                ; \ set up loop:
                    LDA $1662,x             ;  | 
                    AND #$20                ;  | if not smushed (1662 & 0x20), go through loop twice
                    BEQ ON_SCREEN_LOOP      ;  | else, go through loop once
                    INY                     ; / 
ON_SCREEN_LOOP      LDA $D8,x               ; \ 
                    CLC                     ;  | set vertical offscreen if necessary
                    ADC SPR_T1,y            ;  |
                    PHP                     ;  |
                    CMP $1C                 ;  | (vert screen boundry)
                    ROL $00                 ;  |
                    PLP                     ;  |
                    LDA $14D4,x             ;  | 
                    ADC #$00                ;  |
                    LSR $00                 ;  |
                    SBC $1D                 ;  |
                    BEQ ON_SCREEN_Y         ;  |
                    LDA $186C,x             ;  | (vert offscreen)
                    ORA SPR_T2,y            ;  |
                    STA $186C,x             ;  |
ON_SCREEN_Y         DEY                     ;  |
                    BPL ON_SCREEN_LOOP      ; /

                    LDY $15EA,x             ; get offset to sprite OAM
                    LDA $E4,x               ; \ 
                    SEC                     ;  | 
                    SBC $1A                 ;  | $00 = sprite x position relative to screen boarder
                    STA $00                 ; / 
                    LDA $D8,x               ; \ 
                    SEC                     ;  | 
                    SBC $1C                 ;  | $01 = sprite y position relative to screen boarder
                    STA $01                 ; / 
                    RTS                     ; return

INVALID             PLA                     ; \ return from *main gfx routine* subroutine...
                    PLA                     ;  |    ...(not just this subroutine)
                    RTS                     ; /
	

;SUB_OFF_SCREEN
;==============

TABLE3              dcb $40,$B0
TABLE6              dcb $01,$FF 
TABLE4              dcb $30,$C0,$A0,$80,$A0,$40,$60,$B0 
TABLE5              dcb $01,$FF,$01,$FF,$01,$00,$01,$FF

SUB_OFF_SCREEN_X0   LDA #$06                ; \ entry point of routine determines value of $03
                    BRA STORE_03            ;  | 
SUB_OFF_SCREEN_X1   LDA #$04                ;  |
                    BRA STORE_03            ;  |
SUB_OFF_SCREEN_X2   LDA #$02                ;  |
STORE_03            STA $03                 ;  |
                    BRA START_SUB           ;  |
SUB_OFF_SCREEN_HB   STZ $03                 ; /

START_SUB           JSR SUB_IS_OFF_SCREEN   ; \ if sprite is not off screen, return
                    BEQ RETURN_2            ; /    
                    LDA $5B                 ; \  goto VERTICAL_LEVEL if vertical level
                    AND #$01                ;  |
                    BNE VERTICAL_LEVEL      ; /     
                    LDA $D8,x               ; \
                    CLC                     ;  | 
                    ADC #$50                ;  | if the sprite has gone off the bottom of the level...
                    LDA $14D4,x             ;  | (if adding 0x50 to the sprite y position would make the high byte >= 2)
                    ADC #$00                ;  | 
                    CMP #$02                ;  | 
                    BPL ERASE_SPRITE        ; /    ...erase the sprite
                    LDA $167A,x             ; \ if "process offscreen" flag is set, return
                    AND #$04                ;  |
                    BNE RETURN_2            ; /
                    LDA $13                 ; \ 
                    AND #$01                ;  | 
                    ORA $03                 ;  | 
                    STA $01                 ;  |
                    TAY                     ; /
                    LDA $1A                 ;x boundry ;A:0101 X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdizcHC:0256 VC:090 00 FL:16953
                    CLC                     ;A:0100 X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdiZcHC:0280 VC:090 00 FL:16953
                    ADC TABLE4,y            ;A:0100 X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdiZcHC:0294 VC:090 00 FL:16953
                    ROL $00                 ;A:01C0 X:0006 Y:0001 D:0000 DB:03 S:01ED P:eNvMXdizcHC:0326 VC:090 00 FL:16953
                    CMP $E4,x               ;x pos ;A:01C0 X:0006 Y:0001 D:0000 DB:03 S:01ED P:eNvMXdizcHC:0364 VC:090 00 FL:16953
                    PHP                     ;A:01C0 X:0006 Y:0001 D:0000 DB:03 S:01ED P:eNvMXdizCHC:0394 VC:090 00 FL:16953
                    LDA $1B                 ;x boundry hi ;A:01C0 X:0006 Y:0001 D:0000 DB:03 S:01EC P:eNvMXdizCHC:0416 VC:090 00 FL:16953
                    LSR $00                 ;A:0100 X:0006 Y:0001 D:0000 DB:03 S:01EC P:envMXdiZCHC:0440 VC:090 00 FL:16953
                    ADC TABLE5,y            ;A:0100 X:0006 Y:0001 D:0000 DB:03 S:01EC P:envMXdizcHC:0478 VC:090 00 FL:16953
                    PLP                     ;A:01FF X:0006 Y:0001 D:0000 DB:03 S:01EC P:eNvMXdizcHC:0510 VC:090 00 FL:16953
                    SBC $14E0,x             ;x pos high ;A:01FF X:0006 Y:0001 D:0000 DB:03 S:01ED P:eNvMXdizCHC:0538 VC:090 00 FL:16953
                    STA $00                 ;A:01FE X:0006 Y:0001 D:0000 DB:03 S:01ED P:eNvMXdizCHC:0570 VC:090 00 FL:16953
                    LSR $01                 ;A:01FE X:0006 Y:0001 D:0000 DB:03 S:01ED P:eNvMXdizCHC:0594 VC:090 00 FL:16953
                    BCC LABEL20             ;A:01FE X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdiZCHC:0632 VC:090 00 FL:16953
                    EOR #$80                ;A:01FE X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdiZCHC:0648 VC:090 00 FL:16953
                    STA $00                 ;A:017E X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdizCHC:0664 VC:090 00 FL:16953
LABEL20             LDA $00                 ;A:017E X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdizCHC:0688 VC:090 00 FL:16953
                    BPL RETURN_2            ;A:017E X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdizCHC:0712 VC:090 00 FL:16953
ERASE_SPRITE        LDA $14C8,x             ; \ if sprite status < 8, permanently erase sprite
                    CMP #$08                ;  |
                    BCC KILL_SPRITE         ; /
                    LDY $161A,x             ;A:FF08 X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdiZCHC:0140 VC:071 00 FL:21152
                    CPY #$FF                ;A:FF08 X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdizCHC:0172 VC:071 00 FL:21152
                    BEQ KILL_SPRITE         ;A:FF08 X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdizcHC:0188 VC:071 00 FL:21152
                    LDA #$00                ; \ mark sprite to come back    A:FF08 X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdizcHC:0204 VC:071 00 FL:21152
                    STA $1938,y             ; /                             A:FF00 X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdiZcHC:0220 VC:071 00 FL:21152
KILL_SPRITE         STZ $14C8,x             ; erase sprite
RETURN_2            RTS                     ; return

VERTICAL_LEVEL      LDA $167A,x             ; \ if "process offscreen" flag is set, return
                    AND #$04                ;  |
                    BNE RETURN_2            ; /
                    LDA $13                 ; \ only handle every other frame??
                    LSR A                   ;  | 
                    BCS RETURN_2            ; /
                    AND #$01                ;A:0227 X:0006 Y:00EC D:0000 DB:03 S:01ED P:envMXdizcHC:0228 VC:112 00 FL:1142
                    STA $01                 ;A:0201 X:0006 Y:00EC D:0000 DB:03 S:01ED P:envMXdizcHC:0244 VC:112 00 FL:1142
                    TAY                     ;A:0201 X:0006 Y:00EC D:0000 DB:03 S:01ED P:envMXdizcHC:0268 VC:112 00 FL:1142
                    LDA $1C                 ;A:0201 X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdizcHC:0282 VC:112 00 FL:1142
                    CLC                     ;A:02BD X:0006 Y:0001 D:0000 DB:03 S:01ED P:eNvMXdizcHC:0306 VC:112 00 FL:1142
                    ADC TABLE3,y            ;A:02BD X:0006 Y:0001 D:0000 DB:03 S:01ED P:eNvMXdizcHC:0320 VC:112 00 FL:1142
                    ROL $00                 ;A:026D X:0006 Y:0001 D:0000 DB:03 S:01ED P:enVMXdizCHC:0352 VC:112 00 FL:1142
                    CMP $D8,x               ;A:026D X:0006 Y:0001 D:0000 DB:03 S:01ED P:enVMXdizCHC:0390 VC:112 00 FL:1142
                    PHP                     ;A:026D X:0006 Y:0001 D:0000 DB:03 S:01ED P:eNVMXdizcHC:0420 VC:112 00 FL:1142
                    LDA.W $001D             ;A:026D X:0006 Y:0001 D:0000 DB:03 S:01EC P:eNVMXdizcHC:0442 VC:112 00 FL:1142
                    LSR $00                 ;A:0200 X:0006 Y:0001 D:0000 DB:03 S:01EC P:enVMXdiZcHC:0474 VC:112 00 FL:1142
                    ADC TABLE6,y            ;A:0200 X:0006 Y:0001 D:0000 DB:03 S:01EC P:enVMXdizCHC:0512 VC:112 00 FL:1142
                    PLP                     ;A:0200 X:0006 Y:0001 D:0000 DB:03 S:01EC P:envMXdiZCHC:0544 VC:112 00 FL:1142
                    SBC $14D4,x             ;A:0200 X:0006 Y:0001 D:0000 DB:03 S:01ED P:eNVMXdizcHC:0572 VC:112 00 FL:1142
                    STA $00                 ;A:02FF X:0006 Y:0001 D:0000 DB:03 S:01ED P:eNvMXdizcHC:0604 VC:112 00 FL:1142
                    LDY $01                 ;A:02FF X:0006 Y:0001 D:0000 DB:03 S:01ED P:eNvMXdizcHC:0628 VC:112 00 FL:1142
                    BEQ LABEL22             ;A:02FF X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdizcHC:0652 VC:112 00 FL:1142
                    EOR #$80                ;A:02FF X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdizcHC:0668 VC:112 00 FL:1142
                    STA $00                 ;A:027F X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdizcHC:0684 VC:112 00 FL:1142
LABEL22             LDA $00                 ;A:027F X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdizcHC:0708 VC:112 00 FL:1142
                    BPL RETURN_2            ;A:027F X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdizcHC:0732 VC:112 00 FL:1142
                    BMI ERASE_SPRITE        ;A:0280 X:0006 Y:0001 D:0000 DB:03 S:01ED P:eNvMXdizCHC:0170 VC:064 00 FL:1195

SUB_IS_OFF_SCREEN   LDA $15A0,x             ; \ if sprite is on screen, accumulator = 0 
                    ORA $186C,x             ;  |  
                    RTS                     ; / return

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; SUB_HORZ_POS
; This routine determines which side of the sprite Mario is on.  It sets the Y register
; to the direction such that the sprite would face Mario
; It is ripped from $03B817
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

SUB_HORZ_POS		LDY #$00
			LDA $94
			SEC
			SBC $E4,x
			STA $0F
			LDA $95
			SBC $14E0,x
			BPL SPR_L16
			INY
SPR_L16			RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; $B829 - vertical mario/sprite position check - shared
; Y = 1 if mario below sprite??
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

                    ;org $03B829 

SUB_VERT_POS        LDY #$00               ;A:25A1 X:0007 Y:0001 D:0000 DB:03 S:01EA P:envMXdizCHC:0130 VC:085 00 FL:924
                    LDA $96                ;A:25A1 X:0007 Y:0000 D:0000 DB:03 S:01EA P:envMXdiZCHC:0146 VC:085 00 FL:924
                    SEC                    ;A:2546 X:0007 Y:0000 D:0000 DB:03 S:01EA P:envMXdizCHC:0170 VC:085 00 FL:924
                    SBC $D8,x              ;A:2546 X:0007 Y:0000 D:0000 DB:03 S:01EA P:envMXdizCHC:0184 VC:085 00 FL:924
                    STA $0F                ;A:25D6 X:0007 Y:0000 D:0000 DB:03 S:01EA P:eNvMXdizcHC:0214 VC:085 00 FL:924
                    LDA $97                ;A:25D6 X:0007 Y:0000 D:0000 DB:03 S:01EA P:eNvMXdizcHC:0238 VC:085 00 FL:924
                    SBC $14D4,x            ;A:2501 X:0007 Y:0000 D:0000 DB:03 S:01EA P:envMXdizcHC:0262 VC:085 00 FL:924
                    BPL LABEL11            ;A:25FF X:0007 Y:0000 D:0000 DB:03 S:01EA P:eNvMXdizcHC:0294 VC:085 00 FL:924
                    INY                    ;A:25FF X:0007 Y:0000 D:0000 DB:03 S:01EA P:eNvMXdizcHC:0310 VC:085 00 FL:924
LABEL11             RTS                    ;A:25FF X:0007 Y:0001 D:0000 DB:03 S:01EA P:envMXdizcHC:0324 VC:085 00 FL:924    